/* extractLOR.cpp
 *
 * ROOT MACRO
 * 
 * Extract the LOR from the ROOT output
 *
 */

{
	gROOT->Reset();
	
#include "Riostream.h"
#include "math.h"
	
	Int_t i, nevents;	//index i and number of event
	Float_t         globalPosX1, globalPosX2, globalPosY1, globalPosY2, globalPosZ1, globalPosZ2;
	Float_t         sourcePosX1, sourcePosX2, sourcePosY1, sourcePosY2, sourcePosZ1, sourcePosZ2;
	Double_t        time1, time2, deltaTime;
	Int_t			comptonPhantom1, comptonCrystal1, RayleighPhantom1, RayleighCrystal1;
	Int_t			comptonPhantom2, comptonCrystal2, RayleighPhantom2, RayleighCrystal2;
	Int_t			rsectorID1, moduleID1, crystalID1, rsectorID2, moduleID2, crystalID2;
	ofstream fout;	//output file
	fout.open("./moveXYZ-(120,80,45)_2.5uCi.txt");	//store the LOR in the file lor.txt
	
	TFile *f = new TFile("./moveXYZ-(120,80,45)_2.5uCi.root");	//replace "output.root" by your root file
	TTree *t = f->Get("Coincidences");	//this is where you tell it what "tree" you want to access: coincidences, singles, or I don't know what else
	
	//here, you get the address of all the info you want (but you don't get the actual values yet)
	//Coincidences->SetBranchStatus("*",0);
	Coincidences->SetBranchAddress("globalPosX1",&globalPosX1);
	Coincidences->SetBranchAddress("globalPosX2",&globalPosX2);
	Coincidences->SetBranchAddress("globalPosY1",&globalPosY1);
	Coincidences->SetBranchAddress("globalPosY2",&globalPosY2);
	Coincidences->SetBranchAddress("globalPosZ1",&globalPosZ1);
	Coincidences->SetBranchAddress("globalPosZ2",&globalPosZ2);
	Coincidences->SetBranchAddress("sourcePosX1",&sourcePosX1);
	Coincidences->SetBranchAddress("sourcePosX2",&sourcePosX2);
	Coincidences->SetBranchAddress("sourcePosY1",&sourcePosY1);
	Coincidences->SetBranchAddress("sourcePosY2",&sourcePosY2);
	Coincidences->SetBranchAddress("sourcePosZ1",&sourcePosZ1);
	Coincidences->SetBranchAddress("sourcePosZ2",&sourcePosZ2);
	Coincidences->SetBranchAddress("time1",&time1);
	Coincidences->SetBranchAddress("time2",&time2);
	Coincidences->SetBranchAddress("comptonPhantom1",&comptonPhantom1);
	Coincidences->SetBranchAddress("comptonCrystal1",&comptonCrystal1);
	Coincidences->SetBranchAddress("RayleighPhantom1",&RayleighPhantom1);
	Coincidences->SetBranchAddress("RayleighCrystal1",&RayleighCrystal1);
	Coincidences->SetBranchAddress("comptonPhantom2",&comptonPhantom2);
	Coincidences->SetBranchAddress("comptonCrystal2",&comptonCrystal2);
	Coincidences->SetBranchAddress("RayleighPhantom2",&RayleighPhantom2);
	Coincidences->SetBranchAddress("RayleighCrystal2",&RayleighCrystal2);
	Coincidences->SetBranchAddress("rsectorID1",&rsectorID1);
	Coincidences->SetBranchAddress("moduleID1",&moduleID1);
	Coincidences->SetBranchAddress("crystalID1",&crystalID1);
	Coincidences->SetBranchAddress("rsectorID2",&rsectorID2);
	Coincidences->SetBranchAddress("moduleID2",&moduleID2);
	Coincidences->SetBranchAddress("crystalID2",&crystalID2);

	/*NOTE: the "name" of the info you want (e.g. "globalPosX1", or "time1") can be found in the root file;
	 1) Open the root file in root: root yourRootFile.root
	 2) Type: TBrowser t
	 3) Navigate to your root file and open it
	 4) Go into the Coincidences directory ("tree", in the root lingo)
	 5) All the "leaves" you see are the information stored for each coincidence; this is where the "globalPosX1" and "time1" come from
	 */
	
	nevents = t->GetEntries();	//I need the total number of events in order to loop adequately over all events
	cout << "Number of events = " << nevents << "\n";
	
	Int_t numScatter = 0;
	//it's in this loop that you actually extract all the info that you just told root you wanted
	for (i = 0; i < nevents; i++) {
		t->GetEntry(i);	//here, you're accessing event #i; all your variables from above now get the values desired
		
		deltaTime = time2 - time1;
		
		deltaTime /= (1.e-12);
		
		Int_t deltaTimeBinned = (Int_t) deltaTime / 89;
		if (deltaTime < 0) {
			deltaTimeBinned = deltaTimeBinned - 1;
		}

		//cout << "Global position:\n";
		//fout << globalPosX1 << "\t" << globalPosY1 << "\t" << globalPosZ1 << "\t";
		
		//cout << "Discretized position:\n";
		
		//scanner parameters
		double effectiveRingRadius = 413.35;
		int numRsector = 32;
		int numAxialModule = 4;
		int numTransaxialModule = 2;
		int numAxialXtal = 6;
		int numTransaxialXtal = 9;
		double axialXtalPitch = 6.354;
		double transaxialXtalPitch = 4.23375;
		double axialModuleDimension = 38.05;
		double axialModuleGap = 1.5;
		double transaxialModuleDimension = 38.05;
		double transaxialModuleGap = 1.5;
		double modulePitch = 39.55;
		double rsectorPitch = 2*3.14159265359 / numRsector;
		
		//find coordinates relative to xtal array with origin at centre of array
		int j = crystalID1 % numTransaxialXtal;
		int k = (int) (crystalID1 / numTransaxialXtal);
		double x = effectiveRingRadius;
		double y = -((int)numTransaxialXtal/2)*transaxialXtalPitch + j*transaxialXtalPitch;
		double z = -(numAxialXtal/2-0.5)*axialXtalPitch + k*axialXtalPitch;
		
		//cout << "crystalID1 = " << crystalID1 << "\n";
		//cout << "j = " << j << "\tk = " << k << "\n";
		//cout << "x = " << x << "\ty = " << y << "\tz = " << z << "\n";
		
		//Find coordinates relative to detector module, with origin at centre of module, and crystal assumed to be in module #0
		double yModule0 = y - (1./2)*transaxialModuleDimension - (1./2)*transaxialModuleGap;
		double zModule0 = z - (1./2)*axialModuleDimension - (1./2)*axialModuleGap - modulePitch;
		//cout << "yModule0 = " << yModule0 << "\tzModule0 = " << zModule0 << "\n";
		
		//Shift to the correct module
		double yCorrectModule = yModule0 + (moduleID1 % numTransaxialModule)*modulePitch;
		double zCorrectModule = zModule0 + ((int) moduleID1 / numTransaxialModule)*modulePitch;
		//cout << "moduleID1 = " << moduleID1 << "\n";
		//cout << "yCorrectModule = " << yCorrectModule << "\tzCorrectModule = " << zCorrectModule << "\n";
		
		//Rotate to the correct rsector
		double xFinal1 = x*cos(rsectorID1*rsectorPitch) - yCorrectModule*sin(rsectorID1*rsectorPitch);
		double yFinal1 = x*sin(rsectorID1*rsectorPitch) + yCorrectModule*cos(rsectorID1*rsectorPitch);
		double zFinal1 = zCorrectModule;
		//cout << "rsectorID1 = " << rsectorID1 << "\n";
		//cout << "xFinal = " << xFinal << "\tyFinal = " << yFinal << "\n";
		
		//repeat for the second event in the coincidence pair
		//find coordinates relative to xtal array with origin at centre of array
		j = crystalID2 % numTransaxialXtal;
		k = (int) (crystalID2 / numTransaxialXtal);
		x = effectiveRingRadius;
		y = -((int)numTransaxialXtal/2)*transaxialXtalPitch + j*transaxialXtalPitch;
		z = -(numAxialXtal/2-0.5)*axialXtalPitch + k*axialXtalPitch;
		
		//cout << "crystalID1 = " << crystalID1 << "\n";
		//cout << "j = " << j << "\tk = " << k << "\n";
		//cout << "x = " << x << "\ty = " << y << "\tz = " << z << "\n";
		
		//Find coordinates relative to detector module, with origin at centre of module, and crystal assumed to be in module #0
		yModule0 = y - (1./2)*transaxialModuleDimension - (1./2)*transaxialModuleGap;
		zModule0 = z - (1./2)*axialModuleDimension - (1./2)*axialModuleGap - modulePitch;
		//cout << "yModule0 = " << yModule0 << "\tzModule0 = " << zModule0 << "\n";
		
		//Shift to the correct module
		yCorrectModule = yModule0 + (moduleID2 % numTransaxialModule)*modulePitch;
		zCorrectModule = zModule0 + ((int) moduleID2 / numTransaxialModule)*modulePitch;
		
		//Rotate to the correct rsector
		double xFinal2 = x*cos(rsectorID2*rsectorPitch) - yCorrectModule*sin(rsectorID2*rsectorPitch);
		double yFinal2 = x*sin(rsectorID2*rsectorPitch) + yCorrectModule*cos(rsectorID2*rsectorPitch);
		double zFinal2 = zCorrectModule;

		fout << xFinal1 << "\t" << yFinal1 << "\t" << zFinal1 << "\t";
		fout << xFinal2 << "\t" << yFinal2 << "\t" << zFinal2 << "\t";
		fout << time1*1000. << "\t" << deltaTimeBinned << "\n";

		
		/*
		deltaTime = time2 - time1;
		deltaTime /= (1.e-12);
		
		Int_t deltaTimeBinned = (Int_t) deltaTime / 89;
		if (deltaTime < 0) {
			deltaTimeBinned = deltaTimeBinned - 1;
		}
		//cout << "deltaTime = " << deltaTime << "\n";
		//cout << "deltaTimeBinned = " << deltaTimeBinned << "\n";
		
		//if ( comptonPhantom1 > 0 || RayleighPhantom1 > 0
		//	|| comptonPhantom2 > 0 || RayleighPhantom2 > 0) {
		if ( comptonPhantom1 > 0 || comptonPhantom2 > 0 ) {
			++numScatter;
		}
		
		double xCentre, yCentre, zCentre;
		xCentre = (globalPosX1 + globalPosX2)/2;
		yCentre = (globalPosY1 + globalPosY2)/2;
		zCentre = (globalPosZ1 + globalPosZ2)/2;
		
		double dx, dy, dz;
		dx = globalPosX1 - globalPosX2;
		dy = globalPosY1 - globalPosY2;
		dz = globalPosZ1 - globalPosZ2;
		
		double norm = sqrt(dx*dx + dy*dy + dz*dz);
		dx /= norm;
		dy /= norm;
		dz /= norm;
		
		double c = 299792458.;
		double annihilationDist = deltaTime*(1.e-12)*c*1000. / 2;
		double annihilationPtX = xCentre + (annihilationDist * dx);
		double annihilationPtY = yCentre + (annihilationDist * dy);
		double annihilationPtZ = zCentre + (annihilationDist * dz);
		
		double annihilationDistBin = deltaTimeBinned*89.*(1.e-12)*c*1000. / 2;
		double annihilationPtXBin = xCentre + (annihilationDistBin * dx);
		double annihilationPtYBin = yCentre + (annihilationDistBin * dy);
		double annihilationPtZBin = zCentre + (annihilationDistBin * dz);

		
		//write these values to a .txt file
		
		fout << globalPosX1 << "\t" << globalPosY1 << "\t" << globalPosZ1 << "\t";
		fout << globalPosX2 << "\t" << globalPosY2 << "\t" << globalPosZ2 << "\t";
		fout << comptonPhantom1 << "\t" << RayleighPhantom1 << "\t";
		fout << comptonPhantom2 << "\t" << RayleighPhantom2 << "\t";
		fout << time1 << "\t" << time2 << "\t";
		fout << deltaTime << "\t" << deltaTimeBinned << "\t";
		fout << annihilationPtX << "\t" << annihilationPtY << "\t" << annihilationPtZ << "\t";
		fout << annihilationPtXBin << "\t" << annihilationPtYBin << "\t" << annihilationPtZBin << "\n";
		 
		fout << globalPosX1 << "\t" << globalPosY1 << "\t" << globalPosZ1 << "\t";
		fout << globalPosX2 << "\t" << globalPosY2 << "\t" << globalPosZ2 << "\t";
		fout << time1*1000. << "\t" << deltaTimeBinned << "\n";
		*/
		
		//fout << time1 * 1000. << "\t";
		//fout << deltaTimeBinned << "\n";
		//fout << deltaTime << "\n";
	}
	
		//cout << "Scatter fraction = " << (double)numScatter / nevents << "\n";
		
	fout.close();
}